/*
 * m_SegSVM_CCCP.cpp
 *
 * Created on: Aug 1, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */

#ifndef M_SEGSVM_CCCP
#define M_SEGSVM_CCCP

#include "m_utils.h"
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ilcplex/ilocplex.h>

using namespace std;

/* Update constraints for positive signals */
void updatePosConstrs(double *posTrFeatPtr, int nPos, int nBin, IloEnv env,
      IloRangeArray posConstrs, IloNumVarArray xw, IloNumVarArray xalphas, IloNumVar xb, bool useBias){
   double *newPosTrFeatPtr = posTrFeatPtr;
   // Update constraints for new fg segments
   for (int i=0; i < nPos; i++){
      IloExpr posConstrExp(env);
      for (int k=0; k<nBin; k++) posConstrExp += (*newPosTrFeatPtr++)*xw[k];
      //modify the constraint, no need for removing/adding operation.
      //This function will notify the model about the change
      if (useBias) posConstrs[i].setExpr(posConstrExp + xb + xalphas[i]);
      else posConstrs[i].setExpr(posConstrExp + xalphas[i]);
      posConstrExp.end();
   }
}

class SegSVM_OptRslt{
public:
   double *w, b, **objVals, **wDeviation;
   int nIter1, *nCGIters;
   //SegSVM_OptRslt(){};
};

/*
 * Segment based SVM, optimization with Concave Convex Constrain Programming
 * \param featGen: a feature generation object, that supports three methods:
 *    + getInitNegFeats
 *    + getInitPosTrFeats
 *    + getNegFeats
 *    + getPosFeats
 * \param C: C parameter of SVM
 * \param nPos: # of positive instances
 * \param nNeg: # of negative instances
 * \param nBin: # of dim of the weight vector (# of histogram bins)
 * \param runIDString: some string to be added to printing messages for discrimination
 */
template<class Feat>
SegSVM_OptRslt SegSVM_CCCP(Feat featGen, double C, int nPos, int nNeg, int nBin, bool useBias, char *runIDString){
   /*Internal Parameters*/
   int MAX_ITERS_1 = 50; // no of iters for coordinate descent
   int MAX_ITERS_2 = 100;// no of iters for constraint generation
   int NO_INIT_NEG = 30; // no of initial negative constraints for each neg. sample
   int VERBOSE = 2;
   double EXTRA_VIOLATION_TOLFAC = 0.001; // stopping condition for extra violation (inner loop)
   double OBJ_REDUCTION_TOLFAC = 0.0001; // stopping condition for obj. reduction (outer loop)

   double deviation;

   clock_t curTime   = time(NULL);
   clock_t startTime = curTime;

   IloEnv env;
   SegSVM_OptRslt optRslt;
   try{
      mexPrintf("Cplex: Initializing and setting variables\n");
      IloModel mod(env);
      IloCplex cplex(mod); // initialize Cplex

      //variables
      IloNumVarArray xw(env, nBin, -IloInfinity, IloInfinity);
      IloNumVarArray xalphas(env, nPos, 0.0, IloInfinity);
      IloNumVarArray xbetas(env, nNeg, 0.0, IloInfinity);
      IloNumVar      xb(env, -IloInfinity, IloInfinity);

      //associated values of the above variables
      IloNumArray w(env);
      IloNumArray alphas(env);
      IloNumArray betas(env);
      IloNum b;

      // IloRangeArray posConstrs(env, nPos); //should not be used, empty handles to elements
      IloRangeArray posConstrs(env, nPos, 1, IloInfinity);

      for (int i = 0; i < nBin; i++) w.add(0);
      for (int i = 0; i < nNeg; i++) betas.add(0);
      b = 0;

      mexPrintf("Cplex: adding the objective\n");
      IloExpr objExp(env);
      for (int i=0; i < nBin; i++) objExp += 0.5*xw[i]*xw[i];
      for (int i=0; i < nPos; i++) objExp += C/nPos*xalphas[i];
      for (int i=0; i < nNeg; i++) objExp += C/nNeg*xbetas[i];
      mod.add(IloMinimize(env, objExp));
      objExp.end();

      mexPrintf("Cplex: adding place holder for positive constraint\n");
      mod.add(posConstrs);

      // valid entry: CPX_ALG_AUTOMATIC, CPX_ALG_PRIMAL, CPX_ALG_DUAL, CPX_ALG_BARRIER
      int solverAlgo = CPX_ALG_DUAL;
      int solverVerbose = 0; //0: none, 1: iter info, 2: diagnostic info
      cplex.setParam(IloCplex::RootAlg, solverAlgo);
      cplex.setParam(IloCplex::BarDisplay, solverVerbose);
      cplex.setParam(IloCplex::SimDisplay, solverVerbose);

      int *nIters = new int[MAX_ITERS_1]; // keep track of inner iters per outer iter.
      double *w_iter1 = new double[nBin];
      double *w_iter2 = new double[nBin];
      double **objValues  = buildMatrix<double>(MAX_ITERS_1, MAX_ITERS_2);
      double **wDeviation = buildMatrix<double>(MAX_ITERS_1, MAX_ITERS_2);

      int nIter1 = 0;

      double *initPosTrFeatPtr = featGen.getInitPosTrFeats();
      updatePosConstrs(initPosTrFeatPtr, nPos, nBin, env, posConstrs, xw, xalphas, xb, useBias);

      for (int iter1=0; iter1 < MAX_ITERS_1; iter1++){// Coordinate descent
         nIter1++;

         mexPrintf("Cplex: adding initial negative constraints\n");
         IloConstraintArray initNegConstrs(env);
         // Generate initial negative constraints
         for(int i = 0;i < NO_INIT_NEG;i++){
            double *initNegTrFeatPtr = featGen.getInitNegFeats();
            for(int j = 0;j < nNeg;j++){
               IloExpr negConstrExp(env);
               for(int k = 0;k < nBin;k++) negConstrExp += (*initNegTrFeatPtr++)*xw[k];
               if (useBias) initNegConstrs.add(negConstrExp + xb <= -1 + xbetas[j]);
               else initNegConstrs.add(negConstrExp <= -1 + xbetas[j]);
               negConstrExp.end();
            }
         }

         int nNegConstr_total = NO_INIT_NEG*nNeg;
         mod.add(initNegConstrs);
         IloConstraintArray negConstrs(env);
         int negConstrIdx = 0;
         double curObjVal = 0;
         for(int iter2 = 0;iter2 < MAX_ITERS_2;iter2++){
            nIters[iter1] = iter2+1;
            //inner loop, constraint generation
            mexPrintf("%s, C: %g, outer iter: %d, inner iter: %d\n", runIDString, C, iter1 + 1, iter2 + 1);
            startTime = clock();

            // Optimize the problem and obtain solution.
            mexPrintf("Cplex: optimizing... "); startTime = clock();
            if ( !cplex.solve() ) {
               env.error() << "Failed to optimize QP" << endl;
               throw(-1);
            }

            curObjVal = cplex.getObjValue();
            objValues[iter1][iter2] = curObjVal;
            env.out() << "status = " << cplex.getStatus() <<
            ", elapse time: " << diffclock(clock(), startTime) << endl;

            cplex.getValues(w, xw);
            cplex.getValues(betas, xbetas);
            if (useBias) b = cplex.getValue(xb);

            bool isDevEnough = isDeviateEnough(w, w_iter2, &deviation);
            wDeviation[iter1][iter2] = deviation;


            if (VERBOSE >= 3){
               env.out() << "w        = " << w     << endl;
               env.out() << "betas    = " << betas << endl;
               env.out() << "b        = " << b     << endl;
            } else if (VERBOSE >= 1){
               cplex.getValues(alphas, xalphas);
               double w_L2 = 0;
               for (int i=0; i < nBin; i++) w_L2 += w[i]*w[i];
               double alphas_L1 = 0;
               for (int i=0; i < nPos; i++) alphas_L1 += alphas[i];
               double betas_L1 = 0;
               for (int i=0; i < nNeg; i++) betas_L1 += betas[i];
               mexPrintf("   0.5*w*w: %g, C/nPos*alphas: %g, C/nNeg*betas: %g, obj.val: %g\n",
                     0.5*w_L2, C/nPos*alphas_L1, C/nNeg*betas_L1, curObjVal);
               mexPrintf("   w deviation: %g\n", deviation);
            }

            if (!isDevEnough){
               mexPrintf("   deviation is small, quitting to the outer loop,  final obj. val: %g\n", curObjVal);
               break;
            }


            //find maximum violated constraints
            mexPrintf("   Generating new constraints...");
            for(int i = 0;i < nBin;i++)  w_iter2[i] = w[i]; // remember old weights

            double *newNegTrFeatPtr = featGen.getNegFeats(w);
            double *svmVals_neg = cmpSVMVals(newNegTrFeatPtr, nNeg, nBin,  w_iter2, b);

            int nNewConstr = 0;
            double extraConstrViolation = 0;
            for(int i = 0;i < nNeg;i++){
               if(svmVals_neg[i] > -1 + betas[i]){
                  // violated constraint, add new one.
                  nNewConstr++;
                  extraConstrViolation += (svmVals_neg[i] + 1 - betas[i]);
                  IloExpr negConstrExp(env);
                  for(int k=0;k<nBin;k++) negConstrExp += (*newNegTrFeatPtr++)*xw[k];
                  if (useBias) negConstrs.add(negConstrExp + xb <= -1 + xbetas[i]);
                  else negConstrs.add(negConstrExp <= -1 + xbetas[i]);
                  negConstrExp.end();
                  mod.add(negConstrs[negConstrIdx]);
                  negConstrIdx++;
               } else { //no new constraint, increment the pointer by nBin
                  newNegTrFeatPtr += nBin;
               }
            }

            delete [] svmVals_neg;
            nNegConstr_total += nNewConstr;
            extraConstrViolation *= C/nNeg;
            mexPrintf("   new/total: %d/%d, elapsed time: %.2f, extra violation: %f\n",
                  nNewConstr, nNegConstr_total, diffclock(clock(), startTime), extraConstrViolation);
            if ((!nNewConstr)||(extraConstrViolation < EXTRA_VIOLATION_TOLFAC*curObjVal)) {
               mexPrintf("   extraConstrViolation is small, exit to outer loop, final obj. val: %g\n", curObjVal);
               break;
            }
         }

         if (!isDeviateEnough(w, w_iter1, &deviation)){
            mexPrintf("===>outer loop, w deviation: %g, deviation is small, quitting\n", deviation);
            break;
         } else if ((iter1 > 0)  &&
               (objValues[iter1][nIters[iter1]-1] >
               (1- OBJ_REDUCTION_TOLFAC)*objValues[iter1-1][nIters[iter1-1]-1])) {
            mexPrintf("===>outer loop, Obj.value does not deviate much, quitting\n");
            break;
         }


         /* Modify the positive constraints*/
         mexPrintf("==================\nFinding new fg segments for positive images\n");
         startTime = clock();

         for(int i = 0;i < nBin;i++)  w_iter1[i] = w[i]; // remember old weights
         double *newPosTrFeatPtr = featGen.getPosFeats(w);
         mexPrintf("   elapsed time: %.2f\n", diffclock(clock(), startTime));

         double *svmVals_pos = cmpSVMVals(newPosTrFeatPtr, nPos, nBin,  w_iter1, b);
         updatePosConstrs(newPosTrFeatPtr, nPos, nBin, env, posConstrs, xw, xalphas, xb, useBias);
         if(VERBOSE >= 2){
            //display ROC and ACC statistics
            double *newNegTrFeatPtr = featGen.getNegFeats(w);

            // calculate svmVals_neg = newNegTrFeats'*w_iter1 + b;
            double *svmVals_neg = cmpSVMVals(newNegTrFeatPtr, nNeg, nBin,  w_iter1, b);

            // call ml_roc
            mxArray *args2[4];
            mxArray *result2[3];
            args2[0] = mxCreateDoubleMatrix(nNeg, 1, mxREAL);
            memcpy(mxGetPr(args2[0]), svmVals_neg, nNeg * sizeof (double));
            args2[1] = mxCreateDoubleMatrix(nPos, 1, mxREAL);
            memcpy(mxGetPr(args2[1]), svmVals_pos, nPos * sizeof (double));
            args2[2] = mxCreateDoubleScalar(0);
            args2[3] = mxCreateString("ACC");
            mexCallMATLAB(3, result2, 4, args2, "ml_roc");
            double rocArea = (double)(*mxGetPr(result2[0]));
            double acc = (double)(*mxGetPr(result2[2]));
            mexPrintf("   rocArea: %g, acc: %g\n", rocArea, acc);
            mxFree(svmVals_neg);
         }
         mxFree(svmVals_pos);

         // Clear b and betas, if not, no more negative constraint is generated
         // This is due to the condition on EXTRA violation. Clearing b and betas
         // will set EXTRA violation to be total violation.
         // w and alphas need not be cleared
         b = 0;
         for (int i = 0; i < nNeg; i++) betas[i] = 0;

         // Remove old negative constraints
         mod.remove(initNegConstrs);
         for (int i=0; i < negConstrIdx; i++) mod.remove(negConstrs[i]);
         negConstrs.endElements();
         initNegConstrs.endElements();
      }

      mexPrintf("m_SegSVM_CCCP: done, some statistics ...\n");
      mexPrintf("   # outer iters: %d, # inner iters:", nIter1);
      for (int i=0; i < nIter1; i++) mexPrintf("%d ", nIters[i]);
      mexPrintf("\n");

      /* Create outputs*/
      optRslt.w = new double[nBin];
      for (int i=0; i < nBin; i++) optRslt.w[i] = w[i];
      optRslt.b = b;
      optRslt.nIter1 = nIter1;
      optRslt.nCGIters = new int[nIter1];
      for (int i=0; i < nIter1; i++) optRslt.nCGIters[i] = nIters[i];
      optRslt.objVals = new double*[nIter1];
      optRslt.wDeviation = new double*[nIter1];
      for (int i=0; i < nIter1; i++){
         int nIter2 = nIters[i];
         optRslt.objVals[i] = new double[nIter2];
         optRslt.wDeviation[i] = new double[nIter2];
         for (int j=0; j < nIter2; j++){
            optRslt.objVals[i][j] = objValues[i][j];
            optRslt.wDeviation[i][j] = wDeviation[i][j];
         }
      }

      //free some memory
      delete [] nIters;
      delete [] w_iter1;
      delete [] w_iter2;
      destroyMatrix<double>(wDeviation, MAX_ITERS_1, MAX_ITERS_2);
      destroyMatrix<double>(objValues, MAX_ITERS_1, MAX_ITERS_2);
   } catch (IloException& e) {
      cerr << "m_SegSVM_CCCP: concert exception caught: " << e << endl;
   } catch (exception& e) {
      cerr << "m_SegSVM_CCCP: standard exception caught: " << e.what() << endl;
   } catch (...) {
      cerr << "m_SegSVM_CCCP: Unknown exception caught." << endl;
   }

   env.end();
   return optRslt;
}
#endif
