function [Feats, classIDs] = m_cmpInitFeats_1D_MC(metaD, posOrNeg)
% Compute initial features
% Inputs:
%   metaD: a structure with the required fields:
%       D: a cell structure containing the data of length n
%       lb: a 1*n class label vector
%       initOption_pos, initOption_neg: options for positive or negative classes. 
%       nBin: number of histogram bin = # of dimension of the SVM weight vector.
%       nClass: number of classes
%   posOrNeg: must be either 'pos' or 'neg'
% Outputs:
%   Feats: nBin*n matrix, where n is the number of training samples
%   classIDs: 1*nTr vector for class IDs. 
%       If 'posOrNeg' is 'pos', this is metaD.lb
%       If 'posOrNeg' is classIDs is a random assignment that different from metaD.lb
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Last modified: 23 Apr 2011

nBin = metaD.nBin;
nSample = metaD.nSample;
nClass = metaD.nClass;

if strcmpi(posOrNeg, 'pos')
    option = metaD.initOption_pos;
    classIDs = metaD.lb;
elseif strcmp(posOrNeg, 'neg')
    option = metaD.initOption_neg;
    classIDs = randi(nClass-1, [1, nSample]);
    idxs = classIDs - metaD.lb >= 0;
    classIDs(idxs) = classIDs(idxs) + 1;
else
    error('m_cmpSegFeats_1D_MC: unrecognized posOrNeg option');    
end;

if strcmpi(option, 'initW')
    initW = metaD.initW;
    [Feats, classIDs] = m_cmpSegFeats_1D_MC(metaD, initW, posOrNeg);
    return
end;
    
Feats = zeros(nBin, nSample);
if strcmpi(option, 'global')
    for i=1:nSample
        signal  = metaD.D{i}; 
        signalL = max(signal(:,1)); % length of signal
        histo = hist(signal(:,2), 1:nBin);
        Feats(:,i) = histo(:)/signalL;
    end;
elseif strcmpi(option, 'random')
%     rand('twister', sum(100*clock));
    for i=1:nSample
        signal  = metaD.D{i}; 
        signalL = max(signal(:,1)); % length of signal

        t1t2 = ml_rand(1,2,1,signalL, 1);
        t1 = min(t1t2);
        t2 = max(t1t2);
        idxs = and(signal(:,1) >= t1, signal(:,2) <= t2);
        histo = hist(signal(idxs,2), 1:nBin);
        Feats(:,i) = histo(:)/signalL;
    end;
end;
