function [Feats, classIDs] = m_cmpSegFeats_1D_MC(metaD, Ws, posOrNeg)
% Do segmentation and compute feature.
% Inputs:
%   metaD: a structure with the required fields:
%       D: a 1*nTr cell structure containing training data
%       lb: label of training data
%       nTr: number of training samples
%       nSeg: number of foreground segments allowed
%       nBin: number of histogram bin = # of dimension of the SVM weight vector.
%       nClass: number of classes
%   Ws: nBin*nClass weight vectors. 
%   posOrNeg: must be either 'pos', 'neg', or 'test'
% Outputs:
%   Feats: nBin*nTr
%   classIDs: 1*nTr vector for class IDs. 
%       If 'posOrNeg' is 'pos', this is metaD.lb
%       If 'posOrNeg' is 'neg', classIDs(i) is the ID of class that gives highest SVM 
%           score and classIDs(i) cannot be the same with metaD.lb(i).
%       If 'posOrNeg' is 'test', classIDs(i) is the ID of class that gives higher SVM
%           score
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Last modified: 23 Apr 2011


if ~strcmpi(posOrNeg, 'pos') && ~strcmpi(posOrNeg, 'neg') && ~strcmpi(posOrNeg, 'test')
    error('m_cmpSegFeats_1D_MC: unrecognized option');
end;

nBin = metaD.nBin;
nClass = metaD.nClass;
nSample  = metaD.nSample;
nSeg = metaD.nSeg;
Feats = zeros(nBin, nSample);
classIDs = zeros(1, nSample);

switch lower(posOrNeg)
    case 'pos'
        for i=1:nSample
            signal  = metaD.D{i}; 
            signalL = max(signal(:,1)); % length of signal
            lb_i = metaD.lb(i);

            featVec = getFeatVec(signal, signalL, Ws(:, lb_i), nSeg, nBin);
            Feats(:,i) = featVec;
        end;
        classIDs = metaD.lb;
    case 'neg'
        classFeats = zeros(nBin, nClass);
        classScores = zeros(nClass, 1);
        for i=1:nSample
            signal  = metaD.D{i}; 
            signalL = max(signal(:,1)); % length of signal
            lb_i = metaD.lb(i);
            
            for j = [1:lb_i-1, lb_i+1:nClass]
                [classFeats(:,j) classScores(j)] = ...
                    getFeatVec(signal, signalL, Ws(:, j), nSeg, nBin);
            end
            classScores(lb_i) = -inf;
            [dc, classIDs(i)] = max(classScores);
            Feats(:,i) = classFeats(:, classIDs(i));
        end
    case 'test'
        classFeats  = zeros(nBin, nClass);
        classScores = zeros(nClass, 1);
        for i=1:nSample
            signal  = metaD.D{i}; 
            signalL = max(signal(:,1)); % length of signal
            for j = 1:nClass
                [classFeats(:,j) classScores(j)] = ...
                    getFeatVec(signal, signalL, Ws(:, j), nSeg, nBin);
            end
            [dc, classIDs(i)] = max(classScores);
            Feats(:,i) = classFeats(:, classIDs(i));
        end
    otherwise
        error('m_cmpSegFeats_1D_MC: unknown posOrNeg option');
end;

% helper function
function [featVec, energy] = getFeatVec(signal, signalL, w, nSeg, nBin)
    Segs = ml_findMulti1DSegs(signal, signalL, -w, nSeg);
    
    fgIdxs = zeros(signalL, 1); % indexes for parts in the foreground
    for j=1:size(Segs,2)
        fgIdxs(Segs(1,j):Segs(2,j)) = 1;
    end;
    fgIdxs = logical(fgIdxs);
    histo = hist(signal(fgIdxs(signal(:,1)),2), 1:nBin);
    featVec = histo(:)/signalL;
    energy = w'*featVec;
