function m_main_falling(nSeg)
% Run experiment on falling/walking accelerometer data
% Positive class: walking people who fall twice
% Negative class: normal walking people
% This function displays what the algorithm discovers as the discriminative patterns.
% Classification accuracy is not that important.
% Inputs:
%   nSeg: number of segments
%   spOffset: subplot offset, for displaying purpose only. Set to 0 if you don't know.
% Minh Hoai Nguyen
% Date: 13 Apr 2011
% By Minh Hoai Nguyen

load('../data/FallTwice.mat', 'TSs', 'lbs');

n =length(TSs);
D = cat(2, TSs{:});
nBin = 20;
if ~exist('nSeg', 'var')
    nSeg = 2;
end;

Centroids = yael_kmeans (single(D), nBin);
Centroids = double(Centroids);

Ds = cell(1, n);
for i=1:n
    n_i = size(TSs{i}, 2);
    Dist = ml_sqrDist(Centroids, TSs{i});
    [dc, I] = min(Dist, [], 1);
    Ds{i} = [1:n_i; I]';
end;

TrPosIms = Ds(1:15);
TrNegIms = Ds(26:35); 
TstPosIms = Ds(16:25); % diff from TrPosIms, what can the algorithm discover?
TstNegIms = Ds(26:35); % same with TrNegIms, classification accuray is not important


TstPosTSs = TSs(16:25);

metaD.C = 0.001;
metaD.nNeg = length(TrNegIms);
metaD.nPos = length(TrPosIms);
metaD.nBin = nBin;
metaD.nSeg = nSeg;
metaD.cmpInitFeats = 'm_cmpInitFeats_1D';
metaD.cmpSegFeats = 'm_cmpSegFeats_1D';
metaD.useBias = 0;
metaD.runID = 'FallTwice';


% Do classification using global statistics
metaD.initOption_pos = 'Global';
metaD.initOption_neg = 'Global';
metaD.PosD = TrPosIms;
metaD.NegD = TrNegIms;
PosTrFeats  = m_cmpInitFeats_1D(metaD, 'pos'); 
NegTrFeats  = m_cmpInitFeats_1D(metaD, 'neg'); 

metaD.PosD = TstPosIms;
metaD.NegD = TstNegIms;
PosTstFeats = m_cmpInitFeats_1D(metaD, 'pos'); 
NegTstFeats = m_cmpInitFeats_1D(metaD, 'neg'); 

nPosTr = length(TrPosIms);
nNegTr = length(TrNegIms);

opts = sprintf('-t 0 -c %g -w1 %g -w-1 %g', metaD.C, 1/nPosTr, 1/nNegTr);
TrD = [PosTrFeats, NegTrFeats];
TrL = [ones(nPosTr,1); -ones(nNegTr,1)];
model = svmtrain(TrL, TrD', opts); 
initW = full(model.SVs)'*model.sv_coef;
initB = -model.rho;


x = initW'*NegTrFeats + initB;
y = initW'*PosTrFeats + initB;
[rocArea, thres, acc] = ml_roc(x, y, 0);
fprintf('Results on train data, rocArea: %g, acc: %g\n', rocArea, acc);  

x = initW'*NegTstFeats + initB;
y = initW'*PosTstFeats + initB;
[rocArea, thres, acc] = ml_roc(x, y, 0);
fprintf('Results on test data, rocArea: %g, acc: %g\n', rocArea, acc);  


% Running SegSVM
metaD.PosD = TrPosIms;
metaD.NegD = TrNegIms;
metaD.initOption_pos = 'initW';
metaD.initOption_neg = 'random';

metaD.initW = initW;

[binW, b] = m_mexLearnSegSVM(metaD); % learn the weights

% Use the learned weights
[Feats, nSegPos] = m_cmpSegFeats_1D(metaD, binW, 'pos'); 
y = binW(:)'*Feats + b;
[Feats, nSegNeg] = m_cmpSegFeats_1D(metaD, binW, 'neg'); 
x = binW(:)'*Feats + b;
[rocArea, thres, acc] = ml_roc(x, y, 0);
fprintf('Results on train data, rocArea: %g, acc: %g, nSegPos: %d, nSegNeg: %d\n', ...
    rocArea, acc, nSegPos, nSegNeg);    

metaD.PosD = TstPosIms;
metaD.NegD = TstNegIms;

[Feats, nSegPos] = m_cmpSegFeats_1D(metaD, binW, 'pos'); 
y = binW(:)'*Feats + b;
[Feats, nSegNeg] = m_cmpSegFeats_1D(metaD, binW, 'neg'); 
x = binW(:)'*Feats + b;
[rocArea, thres, acc] = ml_roc(x, y, 0);
fprintf('Results on test data, rocArea: %g, acc: %g, nSegPos: %d, nSegNeg: %d\n', ...
    rocArea, acc, nSegPos, nSegNeg);    

% Display
too_small_idxs = abs(binW) < 0.001*max(binW);
binW(too_small_idxs) = -eps; % just to discourage large window

nR = 5; nC = 2; clf;
for i=1:10 % display the first 10 only
    signal = TstPosIms{i};
    
    signalL = max(signal(:,1)); % length of signal
    Segs = ml_findMulti1DSegs(signal, signalL, -binW, nSeg);
    subplot(nR, nC, i); hold off;
    
    for j = 1:size(Segs,2)
        rectangle('Position', [Segs(1,j) -2 (Segs(2,j) - Segs(1,j) + 1) 4], 'FaceColor',0.9*ones(3,1)); hold on;
    end;
    TS_i = TstPosTSs{i};
     
    plot(TS_i(1,:), 'r'); hold on;
    plot(TS_i(2,:), 'g');
    plot(TS_i(3,:), 'b');
    axis([0 2000 -2.5 2.5]);
    title(sprintf('(%c) k = %d', 'a' + i - 1, nSeg), 'fontsize', 14);
end;
